#include <iostream.h>
#include <fstream.h>
#include <string>
#include <time.h>
#include <math.h>
#include <string.h>
#include "config.h"


char const TEMPLATE_FILENAME[RAW_FILENAME_LENGTH] = "./tmp/template.html";
char const SILLY_FILENAME[RAW_FILENAME_LENGTH] = "./tmp/silly.dat";

char const MARKER = '%';

char const POT = 'P';
char const AIR = 'A';
char const MISC = 'M';

int process_file(char const  [], int [TOTAL_FIELDS][PACKET_LENGTH]);
void print_temp(double const, const int);
void print_time(void);
void print_water_level(const int);

int const	FAHRENHEIT = 1;
int const	   CELSIUS = 2;
int const 	    KELVIN = 3;
int const	   RANKINE = 4;

// Thermometer Constants
double const		POT_TEMP_C2 = .00273;
double const		POT_TEMP_C1 = -0.84834;
double const		POT_TEMP_C0 = 141;
double const		AIR_TEMP_C2 = .0004264;
double const 		AIR_TEMP_C1 = -0.3694;
double const		AIR_TEMP_C0 = 143.286;

// Water Level Constants
double const	     WATER_LEVEL_C1 = -.00859776;
double const	     WATER_LEVEL_C2 = 13.5048;


// Common functions


int process_file(char const filename[RAW_FILENAME_LENGTH],
 int raw_data[TOTAL_FIELDS][PACKET_LENGTH])  {
	ifstream rawfile;
	int packets_read = 0;

	rawfile.open(filename, ios::in);

	if(rawfile.bad())  {
		cerr << "wrapper: error opening file '" << filename <<  "'" << endl;
		return -1;
	}
	
	while(packets_read < TOTAL_FIELDS &&
         rawfile >> raw_data[packets_read][0]) {

		for(int i = 0; i < PACKET_DATA_LENGTH; i++)  
			rawfile >> raw_data[packets_read][i + 1];

		packets_read++;
	}

	rawfile.close();

	return packets_read;
}


void print_time(void)  {
	time_t now;
	char * strnowp;
	now = time(0);
	// because ctime returns a \n, process first  
	strnowp = ctime(&now);
	int j = 0;
	while(j < 40 && *(strnowp + j) != '\n')  {
		cout << (strnowp + j)[0];
		j++;
	}

}

void print_temp(double const raw_temp, int const temp_scale)  {
	cout.precision(3);
	switch(temp_scale)  {
		case FAHRENHEIT:
			cout << raw_temp << " degrees fahrenheit";
			break;
		case CELSIUS:
			cout << (raw_temp - 32.0) * 5.0 / 9.0 << " degrees celsius"; 
			break;
		case KELVIN:
			cout << (raw_temp - 32.0) * 5.0 / 9.0 + 273 << " kelvin";
			break;
		case RANKINE:
			cout << (raw_temp + 459.67) << " rankine";
			break;
	}
	cout.precision(0);
}

void print_water_level(const int raw_level)  {

	cout << int(WATER_LEVEL_C1 * raw_level + WATER_LEVEL_C2);	

}

