#include "stamp.h"

void stamp::init_port(void)  {
	
	struct termios tios;
	fd = open(PORT, O_RDWR);
	serial_file = fdopen(fd, "a+");
	tcgetattr(fd, &oldtios);

	cfsetospeed(&tios, B9600);
	cfsetispeed(&tios, B9600);

	tios.c_iflag = tios.c_oflag = tios.c_cflag = tios.c_lflag = 0;
	tios.c_iflag = IGNBRK;
	tios.c_cflag = B9600 | CS8 | CREAD;
	tcflush(fd, TCIOFLUSH);

	tcsetattr(fd, TCSANOW, &tios);
}

int stamp::send(int const commandid = CID_READ_SENSORS)  {

	if(VERBOSE == NOISY)
		cout << "Sending Command..." << endl;

	write(fd, &commandid, 1);

// Since we are done sending, we can now try to read the stamp's response

	read_serial();

	return 0;
}


int stamp::read_serial(void)  {

	int dataid = 0;
	int data[PACKET_DATA_LENGTH] = { 0 };
	string dat;
	int foo = 0;

	if(VERBOSE == NOISY)
		cout << "Reading..." << endl;
	do  {

		read(fd, &foo, 1);
		dataid = foo;
		if(VERBOSE == NOISY) 
			cout << "Got a stream: " << dataid << ' ';
			for(int i = 0; i < PACKET_DATA_LENGTH; i++)  {
				read(fd, &foo, 1);
				data[i] = foo;
				if(VERBOSE == NOISY)
					cout << data[i] << ' ';	
			}			
			if(VERBOSE == NOISY)
				cout << endl;

	}  while ( !store(dataid, data) );

	return 0;
}


int stamp::store(int const dataid, int const data[PACKET_DATA_LENGTH])  {

	// Make sure we are receiving a valid DID
	// A non-zero response means that we are getting errors or End_of_stream
	if(dataid == DID_END_OF_STREAM)
		return 1;
	if(dataid >= TOTAL_FIELDS || dataid == 0)  {
		cerr << "Error parsing stream!" << endl;
		/*
		if(!VERBOSE)  {
			ifstream error_file;
			char buffer;
			error_file.open(ERROR_FILE, ios::in);
			while(error_file.read(&buffer, 1))  {
				cout << buffer;
			}
			error_file.close();
		}
		*/
		cout << serial_file << endl;
		return 2;
	}

/* In the future, we could do more extensive processing for different data
types, but we are stuck with an int [3] right now ;)  */

	raw_data[packets_read][0] = dataid;

	for(int i = 0; i < PACKET_DATA_LENGTH; i++) 
		raw_data[packets_read][i + 1] = data[i];
	
	packets_read++;

	return 0;
}

void stamp::powerdown(void)  {
	
	send(CID_POWERDOWN);

}

