#include "comwrapper.h"
#include "wrapper.h"

int
main(int argc, char * argv[])  {
	int raw_data[TOTAL_FIELDS][PACKET_LENGTH] = { 0 };
	int packets_read = 0;
	int temp_scale = FAHRENHEIT;


	if(argc > 1) 
		temp_scale = process_args(argv[1]);

	packets_read = process_file(RAW_FILENAME, raw_data);

	if(packets_read <= 0)  
		return 1;

	output_html(raw_data, packets_read, temp_scale);

	return 0;
}


int output_html(int const raw_data[TOTAL_FIELDS][PACKET_LENGTH],
                int const data_count, int const temp_scale)  {

	ifstream template_file;
	int code;
	char buffer;
	int position;

	Comments sillycomments;

	sillycomments.read_file(SILLY_FILENAME);

	template_file.open(TEMPLATE_FILENAME, ios::in);

	if(template_file.bad())  {
		cout << "<H1>ERROR opening " << TEMPLATE_FILENAME << ". Please email jsmolens@andrew.cmu.edu!</H1>" << endl;
		return 1;
	}
	
	while(template_file.read(&buffer, 1))  {
		if(buffer == MARKER) {
			template_file >> code; 
			position = find_index(code, raw_data, data_count);

			if(position == -1)  
				cout << " ERROR with codes, email jsmolens@andrew.cmu.edu! ";
			else  {
				int tmparr[PACKET_DATA_LENGTH];
				if(position >= 0)  {
					for(int i = 0; i < PACKET_LENGTH; i++)
						tmparr[i] = raw_data[position][i + 1];

					replace_code(code, tmparr, temp_scale);

				}  else  {
					
					

				}

			}

		} else 
			cout << buffer;
	}

	template_file.close();
		
	return 0;
}

int find_index(int const query, int const raw_data[TOTAL_FIELDS][PACKET_LENGTH],
               int const data_count)  {
	
	int i = 0;
	while(i < data_count && raw_data[i][0] != query)
		i++;

	if(i < data_count) 
		return i;

	else  { 
		switch(query)  {
			case MID_DATE_TIME:
			case MID_SILLY_AIR_TEMP:
			case MID_SILLY_POT_TEMP:
			case MID_SILLY_MISC:
				return -2;
		}
		
		return -1;
	}
}

int replace_code(int const code, int const raw_data[PACKET_DATA_LENGTH], int const temp_scale) {

	switch (code) {
		case DID_POT_IN_PLACE:
			if(raw_data[2] == 0)  
				cout << "in place";
			else 
				cout << "out of Mr. Coffee";
			break;

		case DID_FILTER_IN_PLACE:
			if(raw_data[2] == 0)
				cout << "is in place";
			else
				cout << "is probably being washed...";
			break;

		case DID_POT_TEMP:
			print_temp( POT_TEMP_C2 * pow(double(raw_data[2]), 2) + 
				 POT_TEMP_C1 * double(raw_data[2]) + 
				 POT_TEMP_C0, temp_scale);
			break;

		case DID_AIR_TEMP:
			double air_temp;
			air_temp =  256 * raw_data[1] + raw_data[2];
			print_temp(AIR_TEMP_C2 * pow(air_temp, 2) +
				AIR_TEMP_C1 * air_temp + AIR_TEMP_C0, temp_scale);
			break;

		case DID_TIMES_RUN:
			double times_run;
			times_run = 256 * raw_data[1] + raw_data[2];
			cout << times_run;
			break;

		case MID_DATE_TIME:
			print_time();
			break;

		case DID_WATER_LEVEL:
			print_water_level(raw_data[1] * 256 + raw_data[2]);
			break;

		case DID_END_OF_STREAM:
			break;

		}

	return 0;

}


int process_args(char * arg) {
	
	if(!strncmp(arg, "SCALE", 5))
		return FAHRENHEIT;

	switch(arg[6])  {
		case 'F':
			return FAHRENHEIT;
		case 'C':
			return CELSIUS;
		case 'K':
			return KELVIN;
		case 'R':
			return RANKINE;
	}

	return FAHRENHEIT;
}


void ll_Comments::push(const float index, const string comment)  {
	N_Comments * new_node = new N_Comments(index, comment);
	
	if(!head && !tail)  {
		head = new_node;
		tail = new_node;
	}  else if(head->index >= index)  {
		head->prev = new_node;
		new_node->next = head;
		head = new_node;
	}  else if(tail->index <= index)  {
		tail->next = new_node;
		new_node->prev = tail;
		tail = new_node;
	}  else  {

		N_Comments * current = head->next;

		while(index <= current->index)
			current = current->next;

		current->next->prev = new_node;
		new_node->next = current->next;
		new_node->prev = current;
		current->next = new_node;
	}
}

void Comments::read_file(const char * filename)  {
	
	const int COMMENT_LENGTH = 80;
	ifstream infile;
	char buffer;
	float index;
	char code;

	char comment[COMMENT_LENGTH];

	infile.open(filename, ios::in);

	if(infile.bad())
		return;
	
	// Look for the first type code
	while(infile.read(&buffer, 1) && is_valid_code(buffer)) {
		code = buffer;

		do {
			infile.read(&buffer, 1);
		} while(buffer != MARKER);

		infile >> index;
		
		do {
			infile.read(&buffer, 1);
		} while(buffer != MARKER);
	
		infile.read(&buffer, 1);

		int i = 0;
		while(buffer != MARKER && i < COMMENT_LENGTH)  {	
			comment[i] = buffer;
			i++;
		}
		comment[i] = 0;

		switch(code) {
			case AIR:
				airtemp.push(index, comment);
				break;
			case POT:
				pottemp.push(index, comment);
				break;
			case MISC:
				misc.push(index, comment);
				break;
		}

	}

		
};

char Comments::is_valid_code(const char code)  {

	if(code == POT || code == AIR || code == MISC) 
		return code;
	else
		return 0;

}


